/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.admin ;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.StringTokenizer;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.funambol.framework.engine.source.ContentType;
import com.funambol.framework.engine.source.SyncSourceInfo;
import com.funambol.admin.ui.SourceManagementPanel;
import com.funambol.admin.AdminException;


import com.funambol.email.engine.source.EmailSyncSource;
import com.funambol.email.util.Def;


/**
 * This class implements the configuration panel for an EmailSyncSource
 *
 * @version $Id: EmailSyncSourceConfigPanel.java,v 1.5 2006/02/06 08:58:40 gbmiglia Exp $
 */
public class EmailSyncSourceConfigPanel
                              extends SourceManagementPanel
                              implements Serializable {

  // ----------------------------------------------------------------- Constants

  public static final String NAME_ALLOWED_CHARS =
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_."     ;
  public static final String PORT_ALLOWED_CHARS =
    "0123456789"                                                            ;


  // -------------------------------------------------------------- Private data
  /** label for the panel's name */
  public JLabel             panelName = new JLabel();

  /** border to evidence the title of the panel */
  public TitledBorder       titledBorder1;

  public JLabel             nameLabel        = new JLabel     () ;
  public JTextField         nameValue        = new JTextField () ;

  public JLabel             sourceUriLabel   = new JLabel     () ;
  public JTextField         sourceUriValue   = new JTextField () ;

  //--

  public JLabel             sslCheckLabel    = new JLabel     () ;
  public JCheckBox          sslCheckValue    = new JCheckBox  () ;

  public JLabel             inPortLabel      = new JLabel     () ;
  public JTextField         inPortValue      = new JTextField () ;


  public JLabel             langLabel        = new JLabel     () ;
  public JComboBox          langValue        = new JComboBox  () ;


  //--

  public JLabel             encodeLabel      = new JLabel     () ;
  public JCheckBox          encodeValue      = new JCheckBox  () ;

  public JLabel             typeLabel        = new JLabel();
  public JTextField         typeValue        = new JTextField();

  public JLabel             infoTypesLabel     = new JLabel     () ;
  public JTextField         infoTypesValue     = new JTextField () ;

  public JLabel             infoVersionsLabel     = new JLabel     () ;
  public JTextField         infoVersionsValue     = new JTextField () ;

  public JButton            confirmButton    = new JButton    () ;


  // ----------------------------------------------------------- Constructors

  public EmailSyncSourceConfigPanel() {
      init();
  }

  // ----------------------------------------------------------- Private methods

  /**
   * Create the panel
   * @throws Exception if error occures during creation of the panel
   */
  public void init(){
    // set layout
    this.setLayout(null);

    // set properties of label, position and border
    //  referred to the title of the panel
    titledBorder1 = new TitledBorder("");

    panelName.setFont(titlePanelFont);
    panelName.setText("Edit Email-Connector SyncSource");
    panelName.setBounds(new Rectangle(14, 5, 316, 28));
    panelName.setAlignmentX(SwingConstants.CENTER);
    panelName.setBorder(titledBorder1);

    // --------

    sourceUriLabel.setText("Source URI: ");
    sourceUriLabel.setFont(defaultFont);
    sourceUriLabel.setBounds(new Rectangle(14, 60, 150, 19));
    sourceUriValue.setFont(defaultFont);
    sourceUriValue.setBounds(new Rectangle(170, 60, 350, 19));

    nameLabel.setText("Name: ");
    nameLabel.setFont(defaultFont);
    nameLabel.setBounds(new Rectangle(14, 90, 150, 19));
    nameValue.setFont(defaultFont);
    nameValue.setBounds(new Rectangle(170, 90, 350, 19));

    // --------

    sslCheckLabel.setText("SSL: ");
    sslCheckLabel.setFont(defaultFont);
    sslCheckLabel.setBounds(new Rectangle(14, 130, 150, 19));
    sslCheckValue.setSelected(false);
    sslCheckValue.setBounds(new Rectangle(170, 130, 350, 19));

    inPortLabel.setText("Incoming Server Port: ");
    inPortLabel.setFont(defaultFont);
    inPortLabel.setBounds(new Rectangle(14, 160, 150, 19));
    inPortValue.setFont(defaultFont);
    inPortValue.setBounds(new Rectangle(170, 160, 80, 19));

    langLabel.setText("MailBox Language: ");
    langLabel.setFont(defaultFont);
    langLabel.setBounds(new Rectangle(14, 190, 150, 19));
    langValue.setFont(defaultFont);
    langValue.setBounds(new Rectangle(170, 190, 80, 19) );

    // --------

    encodeLabel.setText("Encoded: ");
    encodeLabel.setFont(defaultFont);
    encodeLabel.setBounds(new Rectangle(14, 230, 150, 19));
    encodeValue.setSelected(false);
    encodeValue.setBounds(new Rectangle(170, 230, 350, 19));

    typeLabel.setText("Type: ");
    typeLabel.setFont(defaultFont);
    typeLabel.setBounds(new Rectangle(14, 260, 150, 19));
    typeValue.setFont(defaultFont);
    typeValue.setBounds(new Rectangle(170, 260, 350, 19));

    infoTypesLabel.setText("Supported Types: ");
    infoTypesLabel.setFont(defaultFont);
    infoTypesLabel.setBounds(new Rectangle(14, 290, 150, 19));
    infoTypesValue.setFont(defaultFont);
    infoTypesValue.setBounds(new Rectangle(170, 290, 350, 19));

    infoVersionsLabel.setText("Supported Versions: ");
    infoVersionsLabel.setFont(defaultFont);
    infoVersionsLabel.setBounds(new Rectangle(14, 320, 150, 19));
    infoVersionsValue.setFont(defaultFont);
    infoVersionsValue.setBounds(new Rectangle(170, 320, 350, 19));

    confirmButton.setFont(defaultFont);
    confirmButton.setText("Add");
    confirmButton.setBounds(170, 350, 70, 25);

    confirmButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event ) {
            try {
                validateValues();
                getValues();
                if (getState() == STATE_INSERT) {
                    EmailSyncSourceConfigPanel.this.actionPerformed(new ActionEvent(EmailSyncSourceConfigPanel.this, ACTION_EVENT_INSERT, event.getActionCommand()));
                } else {
                    EmailSyncSourceConfigPanel.this.actionPerformed(new ActionEvent(EmailSyncSourceConfigPanel.this, ACTION_EVENT_UPDATE, event.getActionCommand()));
                }
            } catch (Exception e) {
                notifyError(new AdminException(e.getMessage()));
            }
        }
    });

    // add all components to the panel
    this.add(panelName      , null ) ;
    this.add(sourceUriLabel , null ) ;
    this.add(sourceUriValue , null ) ;
    this.add(nameLabel      , null ) ;
    this.add(nameValue      , null ) ;
    this.add(sslCheckLabel  , null ) ;
    this.add(sslCheckValue  , null ) ;
    this.add(inPortLabel    , null ) ;
    this.add(inPortValue    , null ) ;
    this.add(langLabel      , null ) ;
    this.add(langValue      , null ) ;
    this.add(encodeLabel    , null ) ;
    this.add(encodeValue    , null ) ;
    this.add(typeLabel      , null ) ;
    this.add(typeValue      , null ) ;
    this.add(infoTypesLabel    , null ) ;
    this.add(infoTypesValue    , null ) ;
    this.add(infoVersionsLabel , null ) ;
    this.add(infoVersionsValue , null ) ;
    this.add(confirmButton  , null ) ;

  }

  /**
   *
   */
  public void updateForm() {

      if (!(getSyncSource() instanceof EmailSyncSource)) {
          error("This is not a EmailSyncSource!"  +
          "Unable to process SyncSource values."     );
          return;
      }

      if (getState() == STATE_INSERT) {
          confirmButton.setText("Add");
      } else if (getState() == STATE_UPDATE) {
          confirmButton.setText("Save");
      }

      EmailSyncSource syncSource = (EmailSyncSource) getSyncSource();

      sourceUriValue.setText(syncSource.getSourceURI());

      nameValue.setText(syncSource.getName());

      if (syncSource.isSSL()) {
          sslCheckValue.setSelected ( true  ) ;
      } else {
          sslCheckValue.setSelected ( false ) ;
      }

      inPortValue.setText(syncSource.getInPort());


      langValue.removeAllItems();
      langValue.addItem(Def.LANG_EN);
      langValue.addItem(Def.LANG_IT);
      langValue.addItem(Def.LANG_DE);
      langValue.addItem(Def.LANG_FR);
      if (syncSource instanceof EmailSyncSource) {
          String lang = ((EmailSyncSource) syncSource).getLanguage();
          if        (Def.LANG_EN.equals(lang)) {
              langValue.setSelectedIndex(0);
          } else if (Def.LANG_IT.equals(lang)) {
              langValue.setSelectedIndex(1);
          } else if (Def.LANG_DE.equals(lang)) {
              langValue.setSelectedIndex(2);
          } else if (Def.LANG_FR.equals(lang)) {
              langValue.setSelectedIndex(3);
          }
      }

      if (syncSource.isEncode()) {
          encodeValue.setSelected ( true  ) ;
      } else {
          encodeValue.setSelected ( false ) ;
      }

      typeValue.setText (syncSource.getType());

      SyncSourceInfo info = syncSource.getInfo();
      if (info != null) {
          ContentType[] types = info.getSupportedTypes();

          StringBuffer typesList    = new StringBuffer();
          StringBuffer versionsList = new StringBuffer();

          for (int i=0; ((types != null) && (i<types.length)); ++i) {

              if (typesList.length() > 0) {
                  typesList.append(',');
                  versionsList.append(',');
              }

              typesList.append(types[i].getType());
              versionsList.append(types[i].getVersion());
          }

          infoTypesValue.setText(typesList.toString());
          infoVersionsValue.setText(versionsList.toString());
      }


      if (syncSource.getSourceURI() != null) {
          sourceUriValue.setEditable(false);
      }

  }


  // ----------------------------------------------------------- Private methods

  /**
   * Checks if the values provided by the user are all valid. In caso of errors,
   * a IllegalArgumentException is thrown.
   *
   * @throws IllegalArgumentException if:
   *         <ul>
   *         <li>name, uri, type or directory are empty (null or zero-length)
   *         <li>the types list length does not match the versions list length
   *         </ul>
   */
  private void validateValues() throws IllegalArgumentException {
      String value = null;

      value = sourceUriValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Source URI' cannot be empty. "  +
                "Please provide a SyncSource URI."      );
      }

      value = nameValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Name' cannot be empty. "    +
                "Please provide a SyncSource name." );
      }

      if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
          throw new
            IllegalArgumentException(
                "Only the following characters are "   +
                "allowed for field 'Name': \n"         +
                NAME_ALLOWED_CHARS                     );
      }

      value = inPortValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
          IllegalArgumentException(
              "Field 'Mail Server Port' cannot be empty. "     +
              "Please provide a SyncSource host."  );
      }

      if (!StringUtils.containsOnly(value, PORT_ALLOWED_CHARS.toCharArray())) {
          throw new
            IllegalArgumentException(
                "Only the following characters are " +
                "allowed for field 'Mail Server Port': \n"       +
                PORT_ALLOWED_CHARS                   );
      }

      value = typeValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException(
          "Field 'Type' cannot be empty. Please provide a SyncSource type.");
      }

      int typesCount = new StringTokenizer(infoTypesValue.getText(), ",").countTokens();
      if (typesCount == 0) {
          throw new IllegalArgumentException("Field 'Supported types' cannot be empty. Please provide one or more supported types.");
      }
      if (typesCount != new StringTokenizer(infoVersionsValue.getText(), ",").countTokens()) {
          throw new IllegalArgumentException("Number of supported types does not match number of versions");
      }

  }

  /**
   *
   * Set syncSource properties with the
   * values provided by the user.
   *
   */
  public void getValues() {

        EmailSyncSource syncSource = (EmailSyncSource)getSyncSource();

        syncSource.setSourceURI(sourceUriValue.getText().trim() ) ;

        syncSource.setName(nameValue.getText().trim() ) ;

        if (sslCheckValue.isSelected()) {
            syncSource.setSSL (true  ) ;
        } else {
            syncSource.setSSL (false ) ;
        }

        syncSource.setInPort(inPortValue.getText().trim() ) ;

        syncSource.setProtocol (Def.PROTOCOL_POP3) ;

        if        (langValue.getSelectedIndex() == 0) {
            syncSource.setLanguage (Def.LANG_EN) ;
        } else if (langValue.getSelectedIndex() == 1) {
            syncSource.setLanguage (Def.LANG_IT) ;
        } else if (langValue.getSelectedIndex() == 2) {
            syncSource.setLanguage (Def.LANG_DE) ;
        } else if (langValue.getSelectedIndex() == 3) {
            syncSource.setLanguage (Def.LANG_FR) ;
        }

        if (encodeValue.isSelected()) {
            syncSource.setEncode (true  ) ;
        } else {
            syncSource.setEncode (false ) ;
        }

        syncSource.setType(typeValue.getText().trim());

        StringTokenizer types    = new StringTokenizer(infoTypesValue.getText(),",");
        StringTokenizer versions = new StringTokenizer(infoVersionsValue.getText(), ",");

        ContentType[] contentTypes= new ContentType[types.countTokens()];

        for(int i = 0; i<contentTypes.length; ++i) {
           contentTypes[i] = new ContentType(types.nextToken().trim(),
                                             versions.nextToken().trim());
        }

        syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));

  }

  /**
   *
   *
   * @param msg String
   */
  private void error(String msg) {
      JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
  }

}
