/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.engine.source;

import java.io.Serializable;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.TimeZone;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Date;
import java.util.StringTokenizer;
import java.net.UnknownHostException;
import java.net.InetAddress;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.security.Principal;
import java.security.GeneralSecurityException;
import java.security.Signature;
import java.sql.Timestamp;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.SyncSource;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.Sync4jDevice;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.engine.SyncItemState;
import com.funambol.framework.engine.SyncItemImpl;
import com.funambol.framework.engine.SyncProperty;
import com.funambol.framework.tools.Base64;
import com.funambol.framework.tools.beans.LazyInitBean;
import com.funambol.framework.tools.beans.BeanInitializationException;
import com.funambol.server.config.Configuration;

import com.funambol.email.pdi.parser.XMLMailParser;
import com.funambol.email.pdi.converter.MailToXML;
import com.funambol.email.pdi.mail.*;
import com.funambol.email.exception.*;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;
import com.funambol.email.items.manager.PopEntityManager;
import com.funambol.email.cache.CrcSyncItemInfo;
import com.funambol.email.items.model.SendResult;



/**
 *
 * This class implements common methods of
 * <pre>
 *  EmailIMAPSyncSource
 *  EmailPOPSyncSource
 *  EmailSMTPSyncSource
 * </pre>
 *
 *
 * version $Id: PopEmailSyncSource.java,v 1.2 2006/02/06 09:00:20 gbmiglia Exp $
 *
 */
public final class PopEmailSyncSource
        extends EmailSyncSource
        implements SyncSource, Serializable, LazyInitBean {

    // --------------------------------------------------------------- Constants

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties



    protected PopEntityManager pem = null;

    // ---------------------------------------------------------- Public Methods

    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {
        super.init();
    }

    /**
     *
     * @param syncItemKey SyncItemKey
     * @param syncItem SyncItem
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean mergeSyncItems(SyncItemKey syncItemKey, SyncItem syncItem) throws SyncSourceException {
         return false;
    }



   /**
    *
    * @param syncItem SyncItem
    * @return boolean
    * @throws SyncSourceException
    */
   public boolean isSyncItemInFilterClause (SyncItem syncItem)
       throws SyncSourceException {
         return false;
   }

   /**
    *
    *
    * @param syncItemKey SyncItemKey
    * @return boolean
    * @throws SyncSourceException
    */
   public boolean isSyncItemInFilterClause (SyncItemKey syncItemKey)
       throws SyncSourceException {
         return false;
   }


   /**
    *
    * @param syncItemKey SyncItemKey
    * @param s Timestamp
    * @throws SyncSourceException
    */
   public void removeSyncItem(SyncItemKey syncItemKey, Timestamp s)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EmailIMAPSyncSource removeSyncItem [" + sourceURI + "]");
        }

        String key = syncItemKey.getKeyAsString();
        String FID  = Utility.getFID(key);
        String FMID = Utility.getFMID(key);
        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item in FID: " + FID + " with FMID: " + FMID);
        }


        try {
            this.pem = getPopEntityManager();
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting email: " +
                                           e.getMessage(), e);
        }

        try {
            this.pem.removeItem(FID,FMID);
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing items", e);
        }

   }


   /**
    *
    *
    * @param operation String
    * @param statusCode int
    * @param keys SyncItemKey[]
    */
   public void setOperationStatus(String operation,
                                  int statusCode,
                                  SyncItemKey[] keys){

        if (log.isLoggable(Level.FINEST)) {
            StringBuffer message = new StringBuffer("Received status code '");
            message.append(statusCode).append("' for a '").append(operation).append("'").
                    append(" for this items: ");

            for (int i = 0; i < keys.length; i++) {
                message.append("\n- " + keys[i].getKeyAsString());
            }
            log.finest(message.toString());
        }
    }


    /**
     *
     *
     * @param syncItemKey SyncItemKey
     * @param s Timestamp
     * @param softDelete boolean
     * @throws SyncSourceException
     */
    public void removeSyncItem(SyncItemKey syncItemKey,
                               Timestamp s,
                               boolean softDelete)
       throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("remove item softDelete: " + softDelete);
        }

        if (softDelete){
          // nothing
        } else {
          // if false
          removeSyncItem(syncItemKey, s);
        }

    }

    /**
     * @see SyncSource
     *
     * @param syncItemKey SyncItemKey
     *
     * @return SyncItem
     *
     * @throws SyncSourceException
     */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey)
      throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromId start");
        }

        String   id       = null;
        SyncItem syncItem = null;

        id = syncItemKey.getKeyAsString();

        try {
            this.pem = getPopEntityManager();
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting POP store: " +
                                           e.getMessage(), e);
        }

        Email mail = null;
        try {
            mail = this.pem.getItemByUID(folderFilter,
                                         timeFilter,
                                         sizeFilter,
                                         numBytes,
                                         id);
        } catch (Exception e) {
            throw new SyncSourceException("Error in getting Mail ", e);
        }

        // get item status
        char status = getSyncItemStateFromId(syncItemKey);

        if (mail != null){
           try {
              syncItem = createSyncItem(mail, status);
           } catch (Exception e) {
              throw new SyncSourceException("Error in creating SyncItem ", e);
           }
        }


        return syncItem;

    }

    /**
     *
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getAllSyncItemKeys() throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItems start ");
        }

        SyncItem[] syncItems = null;
        char       status    = SyncItemState.UNKNOWN;
        String[]   mails     = null;

        try {
            this.pem = getPopEntityManager();
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting Pop Manager: " +
                                           e.getMessage(), e);
        }

        try {
            mails = this.pem.getAllItems(folderFilter,
                                         idFilter,
                                         timeFilter);
        } catch (Exception e) {
            throw new SyncSourceException("Error reading getAllSyncItems : " +
                                          e.getMessage(), e);
        }

        SyncItemKey[] syncItemKeys = null;
        if (mails != null){
           int num = mails.length;
           syncItemKeys = new SyncItemKey[num];
           for (int i = 0; i < num; i++){
                syncItemKeys[i] = new SyncItemKey(mails[i]);
           }
        }

        return syncItemKeys;
    }


    /**
     *
     * @return SyncItemInfo[]
     *
     * @throws SyncSourceException
     */
    public CrcSyncItemInfo[] getAllSyncItemInfo() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo start");
        }

        CrcSyncItemInfo[] syncItemInfo = null;

        try {
            this.pem = getPopEntityManager();
        } catch (Exception e) {
            throw new SyncSourceException("Error connecting mails: " +
                                          e.getMessage(), e);
        }

        try {
            syncItemInfo = this.pem.getAllSyncItemInfo(folderFilter,
                                                       idFilter,
                                                       timeFilter);
        } catch (Exception e) {
            throw new SyncSourceException("Error reading mails: " +
                                          e.getMessage(), e);
        }

        return syncItemInfo;
    }

    /**
     *
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem) throws  SyncSourceException {


        // get SyncItem UID
        String key  = (String)syncItem.getKey().getKeyValue();
        String FMID = Utility.getFMID(key);
        String FID = Utility.getFID(key);
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemKeysFromTwin with UID: " + key +
                       " FMID " + FMID + " FID " + FID);
        }


        Email ctmpout = null;

        // search only in inbox folder
        if (FID.equals(Def.FOLDER_INBOX_ID)){

          // get content of the item
          String content    = null;
          try {
              content = Utility.getContentFromSyncItem(syncItem, this.encode);
          } catch (Exception e) {
              throw new SyncSourceException("Error getting the item " +
                                          syncItem, e);
          }

          try {
             this.pem = getPopEntityManager();
          } catch (EmailAccessException e) {
             throw new SyncSourceException("Error opening EmailPopManager ", e);
          }


          Email email = new Email();
          try {
              email = getFoundationMailFromXML(content);
           } catch (EntityException e) {
              throw new SyncSourceException("Error converting foundation: " +
                                            e.getMessage(), e);
           }

           try {
              ctmpout = pem.getItemByWhereClause(FID, FMID, email);
           } catch (EntityException e) {
              throw new SyncSourceException("Error getting twin", e);
           }
        }

        // converts Email in SyncItem
        SyncItemKey[] syncItemKeys = null;
        SyncItem syncItemOut       = null;

        if (ctmpout != null) {
           try {
             syncItemOut = createSyncItem(ctmpout, SyncItemState.NEW);
           } catch (EntityException e) {
             throw new SyncSourceException("Error creating SyncItem", e);
           }
           syncItemKeys = new SyncItemKey[1];
           syncItemKeys[0] = syncItemOut.getKey();
        } else {
           syncItemKeys = new SyncItemKey[0];
        }


        return syncItemKeys;

    }

    /**
     *
     * This method can only send the mail but not
     * append the mail in the outbox and sent folder.
     *
     *
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {

        // get SyncItem LUID
        String LUID = (String) syncItem.getKey().getKeyValue();
        String FID  = Utility.getFID(LUID);
        if (log.isLoggable(Level.FINEST)) {
           log.finest("addSyncItem folder: " + FID + " luid: " + LUID);
        }

        // get content of the item
        String   content      = null;
        try {
            content = Utility.getContentFromSyncItem(syncItem, this.encode);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " + syncItem, e);
        }

        Email ctmpNew = new Email();
        try {
            ctmpNew = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                                           e.getMessage(), e);
        }

        try {
            this.pem = getPopEntityManager();
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                                          e.getMessage(), e);
        }

        // we can't insert an email using POP3 protocol

        // send mail (only for OUTBOX FOLDER)
        if ( FID.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID) )
        {
           try {
               this.pem.sendItem(FID, ctmpNew, this.mswf);
               // message sent without error, set to true the result
               // we don't store the GUID for the endsync method
               // we can't move the item in the Sent folder
               SendResult sr = new SendResult(LUID, LUID, true, true);
               itemsSent.add(sr);
           } catch (Exception e) {
               // message sent with error, set to false the result
               SendResult sr = new SendResult(LUID, LUID, false, true);
               itemsSent.add(sr);
               String errorMessage = "Error sending the item: " + syncItem;
               throw new SyncSourceException(errorMessage, e);
           }
        }


        // return the input syncItem
        return syncItem;
    }


    /**
     *
     *
     *
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem updateSyncItem(SyncItem syncItem) throws SyncSourceException {

        return syncItem;

    }


    // --------------------------------------------------------- Private Methods

    /**
     *
     * Create PopEntityManager istance (if not already exist)
     *
     * @return PopEntityManager
     * @throws EmailAccessException
     */
    private PopEntityManager getPopEntityManager() throws EmailAccessException {
        if (this.pem == null) {
            this.pem = new PopEntityManager(this.mswf);
        }
        return this.pem;
    }

}
