/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.inboxlistener.console;


import java.io.RandomAccessFile;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.DriverManager;
import java.sql.SQLException;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.engine.DBAccount;
import com.funambol.email.inboxlistener.engine.MailboxAccount;

/**
 * Contains methods to access to the DB tables
 *
 * @version $Id: MenuDAO.java,v 1.2 2006/02/15 16:41:51 gbmiglia Exp $
 */
public class MenuDAO {

  // --------------------------------------------------------------- Constants

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);

  /**
   *
   */
  private static final String SQL_SELECT_ACCOUNTS =
        "select a.username, a.password, b.protocol, b.listener_id " +
        "  from fnbl_email_accounts a, fnbl_email_listeners b " +
        " where a.listener_id = b.listener_id " ;

  /**
   *
   */
  private static final String SQL_INSERT_USER_1 =
        "insert into fnbl_email_listeners " +
        " (listener_id, protocol) values (?, ?); " ;

  /**
   *
   */
  private static final String SQL_INSERT_USER_2 =
        "insert into fnbl_email_accounts " +
        " (username, password, listener_id, activation) values (?,?,?,?)";


  /**
   *
   */
  private static final String SQL_DELETE_USER =
        "delete from fnbl_email_accounts where username = ? ";

  // ------------------------------------------------------------ Private data

  /**   */
  private DBAccount dba   = null;


  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public MenuDAO(DBAccount dba) {
     this.dba = dba;
  }




  // ---------------------------------------------------------- Public Methods



  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public List getAccounts () throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         List accounts         = null;

         try {

             connection = getConnection();

             ps = connection.prepareStatement(SQL_SELECT_ACCOUNTS);

             rs = ps.executeQuery();

             String  username   = null;
             String  password   = null;
             boolean isEnable   = true;
             String  protocol   = null;
             String  listenerID = null;

             MailboxAccount ma = null;
             accounts          = new ArrayList();

             while(rs.next()) {
                 username   = rs.getString(1);
                 password   = rs.getString(2);
                 protocol   = rs.getString(3);
                 listenerID = rs.getString(4);

                 ma = new MailboxAccount(username,
                                         password,
                                         isEnable,
                                         protocol,
                                         listenerID);

                 accounts.add(ma);
             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error reading DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (rs != null) {
                     rs.close();
                 }
                 if (ps != null) {
                     ps.close();
                 }
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }


         return accounts;
  }


  /**
   * reset the DB schema
   *
   */
  public void resetDB() throws InboxListenerException {

         Connection connection = null;

         String line = null;

         try {

             connection = getConnection();

             // drop DB
             String dropfile = "../sql/"+dba.getJdbcDBName()+"/drop_schema.sql";
             System.out.println("open file " + dropfile);
             RandomAccessFile dropStream = new RandomAccessFile(dropfile, "r");
             line = dropStream.readLine();
             while (line != null) {
               line = dropStream.readLine();
               if (line != null){
                 if (line.startsWith("drop")){
                    System.out.println(line);
                    try {
                      Statement stmt = connection.createStatement();
                      stmt.executeUpdate(line);
                      stmt.close();
                    } catch (SQLException e){
                      System.out.println("error deleting table");
                    }
                 }
               }
             }
             dropStream.close();

             // create DB
             String createfile = "../sql/"+dba.getJdbcDBName()+"/create_schema.sql";
             System.out.println("open file " + createfile);

             StringBuffer query = new StringBuffer();
             FileInputStream createStream = new FileInputStream(createfile);
             int c = 0;
             while (  (c = createStream.read()) != -1 ) {
                 if ( (char)c != ';' )  {
                    query.append((char)c);
                 } else {
                    System.out.println(query.toString().trim());
                    try {
                      Statement stmt = connection.createStatement();
                      stmt.executeUpdate(query.toString().trim());
                      stmt.close();
                    } catch (SQLException e){
                      System.out.println("error creating table");
                    }
                    query.delete(0, query.length());
                 }
             }
             createStream.close();

         } catch (FileNotFoundException e) {
             throw new InboxListenerException("Error getting script (" +
                                     e.getMessage() + ")");
         } catch (IOException e) {
             throw new InboxListenerException("Error reading script (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }

  }


  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public void insertAccount (MailboxAccount account)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps_1 = null;
         PreparedStatement ps_2 = null;

         try {

             connection = getConnection();

             String listenerID = account.getListenerID();
             String protocol   = account.getProtocol();
             String username   = account.getUser();
             String password   = account.getPassword();
             boolean isEnable  = account.isEnable();
             String activation = null;
             if (isEnable){
                activation = "y";
             } else {
                activation = "n";
             }
             boolean alreadyIn = false;


             // insert listenerID if is not already present
             try {
                ps_1 = connection.prepareStatement(SQL_INSERT_USER_1);
                ps_1.setString(1, listenerID);
                ps_1.setString(2, protocol);
                ps_1.executeUpdate();
             } catch (Exception e){
                alreadyIn = true;
                System.out.println("listener already in the table. Insert only User");
             }

             // insert user
             ps_2 = connection.prepareStatement(SQL_INSERT_USER_2);
             ps_2.setString(1, username);
             ps_2.setString(2, password);
             ps_2.setString(3, listenerID);
             ps_2.setString(4, activation);
             ps_2.executeUpdate();


         } catch (SQLException e) {
             throw new InboxListenerException("Error handling DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (ps_1 != null) {
                     ps_1.close();
                 }
                 if (ps_2 != null) {
                     ps_2.close();
                 }
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }

  }

  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public void deleteAccount (String username)
    throws InboxListenerException {

         Connection connection  = null;
         PreparedStatement ps = null;

         try {
             connection = getConnection();
             ps = connection.prepareStatement(SQL_DELETE_USER);
             ps.setString(1, username);
             ps.executeUpdate();

         } catch (SQLException e) {
             throw new InboxListenerException("Error deleteing account (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (ps != null) {
                     ps.close();
                 }
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }

  }

  // --------------------------------------------------------- Private Methods

  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  private Connection getConnection() throws InboxListenerException {

       Connection conn = null;

       try {
         Class.forName(dba.getJdbcDriver()).newInstance();
         conn = DriverManager.getConnection(dba.getJdbcUrl(),
                                            dba.getJdbcUser(),
                                            dba.getJdbcPassword());
       } catch (ClassNotFoundException e){
         throw new InboxListenerException("Error connecting DB (Class Not Found): (" +
                                e.getMessage() + ")");
       } catch (InstantiationException e){
         throw new InboxListenerException("Error connecting DB (Instantiation Exception): (" +
                                e.getMessage() + ")");
       } catch (IllegalAccessException e){
         throw new InboxListenerException("Error connecting DB (Illegal Access): " +
                                e.getMessage() + ")");
       } catch (SQLException e){
         throw new InboxListenerException("Error connecting DB (SQLException): (" +
                                e.getMessage() + ")" );
       }

       return conn;
  }

}
