/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.inboxlistener.engine;


import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.DriverManager;
import java.sql.SQLException;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;

/**
 * Contains methods to access to the tables with the
 * information about Mail Server Account.
 *
 * @version $Id: ListenerRegistryDAO.java,v 1.2 2006/02/15 16:42:02 gbmiglia Exp $
 */
public class ListenerRegistryDAO {

  // --------------------------------------------------------------- Constants

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);

  /**
   *
   */
  private static final String SQL_SELECT_ACCOUNTS =
        "select a.username, a.password, b.protocol, b.listener_id " +
        "  from fnbl_email_accounts a, fnbl_email_listeners b " +
        " where a.listener_id = b.listener_id " +
        "   and a.listener_id = ? and a.activation = ? " ;



  // ------------------------------------------------------------ Private data

  /**   */
  private DBAccount dba   = null;


  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public ListenerRegistryDAO(DBAccount dba) {
     this.dba = dba;
  }




  // ---------------------------------------------------------- Public Methods



  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public List getAccounts (String listenerId)
    throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         List accounts         = null;
         String activation     = "y" ;

         try {

             connection = getConnection();

             ps = connection.prepareStatement(SQL_SELECT_ACCOUNTS);

             ps.setString(1, listenerId);
             ps.setString(2, activation);

             rs = ps.executeQuery();

             String  username   = null;
             String  password   = null;
             boolean isEnable   = true;
             String  protocol   = null;
             String  listenerID = null;

             MailboxAccount ma = null;
             accounts          = new ArrayList();

             while(rs.next()) {
                 username   = rs.getString(1);
                 password   = rs.getString(2);
                 protocol   = rs.getString(3);
                 listenerID = rs.getString(4);

                 ma = new MailboxAccount(username,
                                         password,
                                         isEnable,
                                         protocol,
                                         listenerID);

                 accounts.add(ma);
             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error reading DataBase (" +
                                     e.getMessage() + ")");
         } catch (InboxListenerException e) {
             throw new InboxListenerException("Error connecting DataBase (" +
                                     e.getMessage() + ")");
         } catch (Exception e) {
             throw new InboxListenerException("Generic Error", e);
         } finally {
             try {
                 if (rs != null) {
                     rs.close();
                 }
                 if (ps != null) {
                     ps.close();
                 }
                 if (connection != null) {
                     connection.close();
                 }
             } catch (SQLException e){
                 log.severe("Error closing DB " + e.getMessage());
                 throw new InboxListenerException("Error closing DB (" +
                                                  e.getMessage() + ")" );
             }
         }


         return accounts;
  }



  // --------------------------------------------------------- Private Methods

  /**
   *
   *
   * @return Connection
   * @throws SeccException
   */
  private Connection getConnection() throws InboxListenerException {

       Connection conn = null;

       try {
         Class.forName(dba.getJdbcDriver()).newInstance();
         conn = DriverManager.getConnection(dba.getJdbcUrl(),
                                            dba.getJdbcUser(),
                                            dba.getJdbcPassword());
       } catch (ClassNotFoundException e){
         throw new InboxListenerException("Error connecting DB (Class Not Found): (" +
                                e.getMessage() + ")");
       } catch (InstantiationException e){
         throw new InboxListenerException("Error connecting DB (Instantiation Exception): (" +
                                e.getMessage() + ")");
       } catch (IllegalAccessException e){
         throw new InboxListenerException("Error connecting DB (Illegal Access): " +
                                e.getMessage() + ")");
       } catch (SQLException e){
         throw new InboxListenerException("Error connecting DB (SQLException): (" +
                                e.getMessage() + ")" );
       }

       return conn;
  }

}
