/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.inboxlistener.engine;


import java.util.List;
import java.util.ArrayList;
import java.util.logging.Logger;
import java.util.logging.Level;

import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;

import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;

import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.InboxListenerException;


/**
 *
 *
 * @version $Id: MailboxListener.java,v 1.1 2006/02/15 13:21:44 gbmiglia Exp $
 */
public class MailboxListener extends Thread {

  //------------------------------------------------------------------ CONSTANTS

  private Logger log = Logger.getLogger(Def.LOGGER_NAME);

  //----------------------------------------------------------------- PROPERTIES

  /** owner of the thread */
  private List messageIDs = new ArrayList();

  /**   */
  private boolean startupNotification = false ;

  /** owner of the thread */
  private MailboxAccount mba;

  /** mail server coords */
  private MSAccount msa;

  /** webservices coords */
  private WSAccount wsa;

  /** to stop the thread  */
  private boolean go = true ;


  //--------------------------------------------------------------- CONSTRUCTORS

  /**
   *
   */
  public MailboxListener(boolean startupNotification,
                         MailboxAccount mba,
                         MSAccount msa,
                         WSAccount wsa) {
     this.startupNotification = startupNotification;
     this.mba                 = mba;
     this.msa                 = msa;
     this.wsa                 = wsa;
  }


  //------------------------------------------------------------- PUBLIC METHODS

  /**
   *
   * @todo
   *
   * this thread is not always connected to the mailbox.
   * every X seconds (interval) it starts a connection and read the ids
   * in the inbox folder; then compares the previous time read ids.
   * If there are new mails call the Sync4j AdminManager web service
   * in order to trigger the push message
   *
   * @todo:
   *  - check SSL
   *  - how choose external mail and sync mail
   *
   *
   */
  public void run() {

     if (log.isLoggable(Level.FINEST)) {
        log.finest("Run MailboxListener for : " + mba.toString() +
                   " interval : " + msa.getInterval() +
                   " startupNotification : " + startupNotification );
     }

     IMailServerWrapper mswf = MailServerWrapperFactory.getMailServerWrapper(
                                            mba.getProtocol());

     Folder  inbox   = null;
     boolean hasNew  = false;
     int     counter = 1;

     while (go) {

        try {

           mswf.openConnection(msa.getOutHost(),
                               msa.getInHost(),
                               msa.getPort()+"",
                               "en",
                               mba.getUser(),
                               mba.getPassword(),
                               msa.getIsSSL());

           inbox = mswf.getStore().getFolder(Def.LABEL_INBOX);
           inbox.open(Folder.READ_ONLY);

           // check if there are new mails
           hasNew = hasNewMessages(inbox);

           if (log.isLoggable(Level.FINEST)) {
              log.finest("MailboxListener for : " + mba.toString() +
                         " has new mail: " + hasNew );
           }

           inbox.close(true);

           mswf.releaseConnection();


        } catch (MessagingException e) {
           e.printStackTrace();
        } catch (EmailAccessException e) {
           e.printStackTrace();
        } catch (InboxListenerException e) {
           e.printStackTrace();
        }


        if (log.isLoggable(Level.FINEST)) {
          log.finest("MailboxListener for : " + mba.toString() +
                     " call Notification Manager: " + hasNew );
        }

        // call notification system
        try {
           if (hasNew){
              if (counter == 1 && !startupNotification){
                // do nothing
              } else {
                NotificationWrapper nw = new NotificationWrapper(mba, wsa);
                nw.callNotification();
              }
           }
        } catch (Exception e){
           e.printStackTrace();
        }

        hasNew  = false;
        counter = 2;

        try{
           sleep(msa.getInterval());
        } catch (Exception e){
           e.printStackTrace();
        }


     }

  }

  /**
   *
   */
  public void stopThread(){
     if (log.isLoggable(Level.FINEST)) {
        log.finest("Shutting down MailboxListener for : " + mba.toString() );
     }
     this.go = false;
  }


  //------------------------------------------------------------ PRIVATE METHODS

  private boolean hasNewMessages(Folder inbox)
    throws InboxListenerException {

     String message_id        = null;
     String syncLabel         = null;
     String message_id_old    = null;
     boolean isAlreadyInInbox = false;
     boolean hasNewEmail      = false;

     try {
        Message[] messages = inbox.getMessages();

        boolean loop = true;
        int     max  = messages.length;
        int     i    = 0;

        if (max > 0){
          while (loop){
            message_id       = Utility.getMessageID(messages[i]);
            syncLabel        = Utility.getSyncLabel(messages[i]);
            isAlreadyInInbox = false;

            if (log.isLoggable(Level.FINEST)) {
               log.finest("Checking message : " + message_id +
                          " - " + syncLabel);
            }

            if (syncLabel == null) {   // external mail
               if (message_id != null){
                  if (log.isLoggable(Level.FINEST)) {
                     log.finest(" start loop on old list per id " + message_id);
                  }
                  for (int y=0; y<messageIDs.size(); y++){
                     message_id_old = (String)messageIDs.get(y);
                     if (log.isLoggable(Level.FINEST)) {
                         log.finest("\n start loop on old list " +
                                    "\n  message_id     " + message_id +
                                    "\n  message_id_old " + message_id_old);
                     }
                     if (message_id_old.equals(message_id)){ // already in the inbox
                        isAlreadyInInbox = true;
                     }
                  }
                  if (log.isLoggable(Level.FINEST)) {
                     log.finest(" end FOR loop ... isAlreadyInInbox value:" + isAlreadyInInbox);
                  }
                  if (!isAlreadyInInbox){
                    hasNewEmail = true;
                    loop = false;
                  }
               }
            } else {
               if (log.isLoggable(Level.FINEST)) {
                  log.finest(" no loop on old list per id " + message_id);
               }
            }

            if (i == max-1){
              loop = false;
            } else {
              i++;
            }

          }
        }

        if (log.isLoggable(Level.FINEST)) {
           log.finest("hasNewEmail --> " + hasNewEmail);
        }

        // replace the new messageids list
        messageIDs.clear();
        for (int y=0; y<messages.length; y++) {
           message_id = Utility.getMessageID(messages[y]);
           if (message_id != null){
              messageIDs.add(message_id);
           }
        }

     } catch (MessagingException me){
        throw new InboxListenerException("Error checking new messages", me);
     }

     return hasNewEmail;
  }

}
