/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.items.dao;


import java.util.Date;
import java.util.logging.Logger;
import java.util.logging.Level;
import javax.mail.Message;
import javax.mail.Flags;
import javax.mail.MessagingException;
import com.sun.mail.pop3.POP3Folder;
import javax.mail.search.ComparisonTerm;
import javax.mail.search.ReceivedDateTerm;
import javax.mail.search.SearchTerm;
import javax.mail.search.MessageIDTerm;

import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.EISMessage;
import com.funambol.email.util.*;


/**
 * This class implements methods to access data
 * in Mail Server datastore.
 *
 * @version $Id: PopEntityDAO.java,v 1.2 2006/02/06 09:01:39 gbmiglia Exp $
 */
public class PopEntityDAO extends EntityDAO {


    // ------------------------------------------------------------ Private data

    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);


    // -------------------------------------------------------------- Properties

    /**
     *
     */
    public PopEntityDAO(){
     }


    // -----------------------------------------------------------Public Methods

    /**
     * gets all Messages
     *
     *
     * @param f POP3Folder
     * @param idFilter String
     * @param timeFilter Date
     * @return Message[]
     * @throws EntityException
     */
    public Message[] getAllItems(POP3Folder f, String idFilter, Date timeFilter)
       throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("DAO start getAllItems ");
        }

        Message[] messages = null;
        Message   msg      = null;

        try {
           if (idFilter != null){
               String uid = Utility.getFMID(idFilter);
               msg = getMessageByUID(f , uid);
               messages = new Message[1];
               messages[0] = msg ;
           } else {
               if (timeFilter != null){
                 SearchTerm st =  new ReceivedDateTerm(ComparisonTerm.GE, timeFilter);
                 messages = f.search(st);
               } else {
                 messages = f.getMessages();
               }
           }

           if (messages != null){
               if (log.isLoggable(Level.FINEST)) {
                  log.finest("number of items " + messages.length);
               }
           }
        } catch (MessagingException me){
           throw new EntityException(me);
        }

        if (log.isLoggable(Level.FINEST)) {
           log.finest("DAO end getAllItems ");
        }

        return messages;

    }
    /**
     *
     *
     * @param f IMAPFolder
     * @param uid long
     * @throws EntityException
     */
    public void removeItem(POP3Folder f, String uid)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP DAO start removePopItem ");
        }

        try {

            Message msg = getMessageByUID(f ,uid);

            if (msg != null){
               msg.setFlag(Flags.Flag.DELETED, true);
            }

        } catch (MessagingException me){
            throw new EntityException(me);
        }

    }


    /**
     * serch a document using UID
     * for POP3  protocol UID is String
     *
     *
     * @param f POP3Folder
     * @param uid String
     *
     * @return Message
     *
     * @throws EntityException
     */
    public Message getItemByUID(POP3Folder f, String uid)
       throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("POPDAO start getItemByUID folder: " + f + " uid: " + uid);
        }


        Message msg = null;
        try {
            msg = getMessageByUID(f,uid);
        } catch (MessagingException me){
            throw new EntityException(me);
        }

        if (log.isLoggable(Level.FINEST)) {
           log.finest("POPDAO end getItemByUID ");
        }

        return msg;
    }

    /**
     * serch a document using the header Message-ID
     *
     *
     * @param f POP3Folder
     * @param FID String
     * @param message_id String
     * @return EISMessage
     * @throws EntityException
     */
    public EISMessage getItemByMessageID(POP3Folder f,
                                         String FID,
                                         String message_id)
       throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("DAO start getItemByMessageID Header Message-ID: " + message_id);
        }
        EISMessage eisMessage = null;
        Message    msg        = null;

        try {
           SearchTerm st =  new MessageIDTerm(message_id);
           Message[] messages = f.search(st);

           if (messages != null){
              if (log.isLoggable(Level.FINEST)) {
                 log.finest("Messages Found Message-ID: " + messages.length);
              }
              if (messages.length > 0){
                 msg = messages[0];
                 String messid = null;
                 String FMID   = null;
                 if (msg != null){
                    messid = Utility.getMessageID(msg);
                    FMID = String.valueOf(f.getUID(msg));
                    if (log.isLoggable(Level.FINEST)) {
                       log.finest(" found Message-ID: " + messid );
                    }
                    if (message_id.equalsIgnoreCase(messid)){
                       if (log.isLoggable(Level.FINEST)) {
                          log.finest("Email already Exist");
                       }
                       eisMessage = new EISMessage(FID, FMID, messid, msg);
                    }
                 }
              }
           }
        } catch (MessagingException me){
            throw new EntityException(me);
        }


        if (log.isLoggable(Level.FINEST)) {
           log.finest("DAO end getItemByMessageID message ");
        }

        return eisMessage;
    }


   //----------------------------------------------------------- PRIVATE METHODS

  /**
   * workaround
   *
   * @param f POP3Folder
   * @param uid String
   * @return Message
   * @throws MessagingException
   */
  private Message getMessageByUID(POP3Folder f, String uid)
     throws MessagingException{
      Message msg = null;

      Message[] messages = f.getMessages();

      int n = messages.length;
      String uidTmp = null;

      for (int i=0; i<n; i++) {
          msg = messages[i];
          uidTmp = f.getUID(msg);

          if (uidTmp != null){
              if (uidTmp.equalsIgnoreCase(uid)) {
                  break;
              }
          }
      }

      return msg;
  }

}
