/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.transport  ;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Locale;
import java.util.Properties;
import java.util.Locale;

import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;
import com.sun.net.ssl.internal.ssl.Provider;

import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.util.Def;


/**
 *
 * <p>
 * Create a connection with a Mail Server
 * </p>
 *
 * @version $Id: CommonMailServerWrapper.java,v 1.2 2006/02/06 09:03:37 gbmiglia Exp $
 */
public abstract class CommonMailServerWrapper
        implements IMailServerWrapper
{

    // --------------------------------------------------------------- Constants

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    protected String RESOURCE_NAME  = "com/funambol/email/resources/Lang";


    // -------------------------------------------------------------- Properties

    /**     */
    protected Session session;
    /**     */
    protected Store store ;
    /**     */
    protected Locale locale;


    /**
     *
     *
     * @param outgoingServer String
     * @param incomingServer String
     * @param incomingPort String
     * @param language String
     * @param user String
     * @param password String
     * @param isSSL boolean
     * @throws EmailAccessException
     */
    public abstract void openConnection(String  outgoingServer,
                                        String  incomingServer,
                                        String  incomingPort,
                                        String  language,
                                        String  user,
                                        String  password,
                                        boolean isSSL)
      throws EmailAccessException ;

    /**
     *
     * @throws EmailAccessException
     */
    public abstract void releaseConnection()  throws EmailAccessException ;

    /**
     *
     *
     *
     * @param outgoingServer String
     * @param incomingServer String
     * @param incomingPort String
     * @param user String
     * @param password String
     * @param isSSL boolean
     * @throws EmailAccessException
     */
    public void checkUser(String  outgoingServer,
                          String  incomingServer,
                          String  incomingPort,
                          String  user,
                          String  password,
                          boolean isSSL)
     throws EmailAccessException{

      if (log.isLoggable(Level.FINEST)) {
          log.finest("\nCONNECTION PARAMs"+
                     "\nin           " + outgoingServer+
                     "\nout          " + incomingServer+
                     "\nport         " + incomingPort+
                     "\nprotocol     " + this.getClass().getName()+
                     "\nuser         " + user+
                     "\nSSL          " + isSSL);
      }

      // contains for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;


      if (isSSL) {
        setConnectionSSL(props,
                         outgoingServer,
                         incomingServer,
                         incomingPort,
                         provider);
      } else {
        setConnectionNormal(props,
                            outgoingServer,
                            incomingServer,
                            incomingPort);
      }

      if (log.isLoggable(Level.FINEST)) {
          log.finest("COMMON property:" + props.getProperty("mail.store.protocol"));
      }

      try{
         this.session = Session.getInstance(props);
         //this.session.setDebug(true);
         this.store = session.getStore();
         this.store.connect(incomingServer, user, password);
      } catch (Exception e) {
         log.severe("Error Opening Connection Store: " + e);
         log.throwing(this.getClass().getName(), "openConnection", e);
	     throw new EmailAccessException("Error Opening Connection Store (" + e.getMessage() + ")");
      }

      // check INBOX folder
      try {
         Folder inbox  = store.getFolder("INBOX");
      } catch (Exception e) {
	    throw new EmailAccessException("Error getting Inbox Folders " +
                                       e.toString());
      }

      // close store
      try {
         this.store.close();
         this.store = null;
         this.session = null;
         props.clear();
      } catch (Exception e) {
	    throw new EmailAccessException("Error closing store " +
                                       e.toString());
      }

    }

    /**
     *
     * @param props Properties
     * @param outgoingServer String
     * @param incomingServer String
     * @param incomingPort String
     */
    protected abstract void setConnectionNormal(Properties props,
                                             String outgoingServer,
                                             String incomingServer,
                                             String incomingPort);

    /**
     *
     * @param props Properties
     * @param outgoingServer String
     * @param incomingServer String
     * @param incomingPort String
     * @param provider Provider
     */
    protected abstract void  setConnectionSSL(Properties props,
                                           String outgoingServer,
                                           String incomingServer,
                                           String incomingPort,
                                           Provider provider);
    /**
     *
     *
     * @return Session
     */
    public Session getSession(){
       return this.session;
    }

    /**
     *
     * @return Store
     */
    public Store getStore(){
       return this.store;
    }


    /**
     *
     *
     * @return Locale
     */
    public Locale getLocale(){
       return this.locale;
    }

}
